// Interest on Reserves -- Non Optimized Version 
// Cash-Credit 3 goods model w/ log-log preferences 
// Modified version of Handbook model and Debt Management Model 
// log(v/v*) version of Tau(v) 
// Rotemberg Pricing with Overlending Externality 
// Transactions costs parameters set with regression 
// Last modified 8/27/2015 
 
//DECLARATIONS. 
//Endogenous variables 
var cm, cd, cc, pi, lambda, ib, id, iL, ir, w, y, del, mh, res, dep, 
Welf, Util, n, tau, vel, 
def, chk1, chk2, chk3, chk4, chk5, chk6, chk7, chk8, z, zL, spread, 
lmult3,  lmult4,  lmult5,  lmult6,  lmult7,  lmult8,  lmult9,   
    lmult10,  lmult11,  lmult12,  lmult13,  lmult14,  lmult15,  lmult16,   
    lmult17,  lmult18; 

//Exogenous variables 
//var z,zL; 
 
varexo eps, epsd; 
 
//PARAMETER VALUES. 
parameters nbeta, phim, phic, phid, sig, chi, rot, rr, ATC, vstar, dels, zLs, theta, thetam,  
pis, irs, ibs, vels, taus, ids, ws, iLs, cms, cds, ccs, ns, ress, lambdas, sub, rcp,
lmult3_SS, lmult4_SS, lmult5_SS, lmult6_SS, lmult7_SS, lmult8_SS, lmult9_SS,
lmult10_SS, lmult11_SS, lmult12_SS, lmult13_SS, lmult14_SS, lmult15_SS, 
lmult16_SS, lmult17_SS, lmult18_SS; 
 
nbeta = 0.99;   
sig = 6;        
rot = 4;    
phim = .15; 
phid = .65; 
phic = 1-phim-phid; 
chi = 1;        
rr = 0.9;       
thetam = 1.5;  
ATC = 1/22.9;    // 1/slope in regression 
vstar = 4; //log of vstar (both vel and vstar are in logs) 
sub = sig/(sig-1); 
theta=0; 
rcp=0.03; 
 
//[gamma]=res_int_gamma(nbeta,sig,rot,phim,phic,phid,chi,ATC,vstar,sub,rcp,theta); 
zLs= 0.01;
// use res_int_gamma rather than set zLs when theta > 0

x0(1) = 1.0125; 
x0(2) = 1.0107; 

f=@(x)res_int_ramsey_ss(x,nbeta,sig,rot,phim,phid,phic,chi,ATC,vstar,zLs,sub,theta,rcp); 

options = optimset('TolFun', 1E-10);
[x,fval,exitflag] = fminsearch(f,x0,options)

[errcheck,lmss_vals,ssv]=res_int_ramsey_ss(x,nbeta,sig,rot,phim,phid,phic,chi,ATC,vstar,zLs,sub,theta,rcp); 

pis=x(1);
irs=x(2); 
ibs = ssv(1); 
vels = ssv(2); 
taus = ssv(3); 
ids = ssv(4); 
ws = ssv(5); 
iLs = ssv(6); 
cms = ssv(7); 
cds = ssv(8); 
ccs = ssv(9); 
ns = ssv(10); 
ress = ssv(11); 
lambdas = ssv(12); 
dels = ssv(13); 

lmult3_SS = lmss_vals(1); 
lmult4_SS = lmss_vals(2); 
lmult5_SS = lmss_vals(3); 
lmult6_SS = lmss_vals(4); 
lmult7_SS = lmss_vals(5); 
lmult8_SS = lmss_vals(6); 
lmult9_SS = lmss_vals(7); 
lmult10_SS = lmss_vals(8); 
lmult11_SS = lmss_vals(9); 
lmult12_SS = lmss_vals(10); 
lmult13_SS = lmss_vals(11); 
lmult14_SS = lmss_vals(12); 
lmult15_SS = lmss_vals(13); 
lmult16_SS = lmss_vals(14); 
lmult17_SS = lmss_vals(15); 
lmult18_SS = lmss_vals(16); 

//zz=[theta,rcp,sub,pis,ibs,irs,ids,iLs,vels,vstar,ns];
//xlswrite('Results_resint_r_ltau.xlsx',zz,'Sheet1','A26');

//MODEL 
model; 
 
//Exogenous Processes 
z = rr*z(-1) + eps; 
zL= log(zLs)*(1-rr) + rr*zL(-1) + epsd; 
 
// Utility and Welfare 
Util = phim*cm + phid*cd + phic*cc - (1/(1+chi))*exp(n)^(1+chi); 
Welf = Util + nbeta*Welf(+1); 
 
//Consumers FOCs 
log(phim) - cm = lambda; 
cc = log(phic/phim) + ib + cm;  
cd = log(phid/phim) + ib - log(1+exp(ib)-exp(id)) + cm; 
w + log(phim) - cm = chi*n; 
1 = nbeta*exp(ib)*exp(lambda(+1)-lambda-pi(+1)); 
 
cm = mh; 
cd = dep; 
 
//Banks and Deposits 
vel = dep-res;      
tau = (ATC/exp(vel))*(exp(vel)/exp(vstar)-1-vel+vstar); 
exp(ib) - exp(ir) = exp(ib)*ATC*(vel - vstar); 
exp(ib) - exp(id) = exp(ib)*ATC*(1/exp(vstar)-1/exp(vel)); 
del = zL + theta*(iL+w+n); 
exp(iL) = exp(ib)/(1-exp(del));  
 
//Production  
exp(y) = exp(z + n)*(1-exp(del)); 
 
//GDP identity 
exp(y)*(1-(rot/2)*(exp(pi)-1)^2)=exp(cm)+exp(cd)*(1+rcp*tau)+exp(cc); 
 
//Rotemberg price setting 
rot*(exp(pi)-1)*exp(pi+y)=(sub*(1-sig)*exp(y)+sig*exp(y+iL+w-z))/exp(ib) 
    + rot*nbeta*exp(lambda(+1)-lambda)*(exp(pi(+1))-1)*exp(pi(+1))*exp(y(+1)); 
 
// Policymaker's First-Order Conditions 
phic+lmult4-lmult17*exp(cc); 
phid+lmult5+lmult9-lmult17*exp(cd)*(1+rcp*tau); 
phim-lmult3-lmult4-lmult5-lmult6+lmult8-lmult17*exp(cm); 
lmult14-lmult15*exp(ib)/(1-exp(del))^2*exp(del)+lmult16*exp(z+n)*exp(del); 
-lmult9-lmult10; 
-lmult4+lmult5*(-1+exp(ib)/(1+exp(ib)-exp(id)))-lmult7*nbeta*exp(ib)*exp(lambda(+1)-lambda-pi(+1))+lmult12*(exp(ib)-exp(ib)*ATC*(vel-vstar))+lmult13*(exp(ib)-exp(ib)*ATC*(1/exp(vstar)-1/exp(vel)))-lmult15*exp(ib)/(1-exp(del))+lmult18*(sub*(1-sig)*exp(y)+sig*exp(y+iL+w-z))/exp(ib); 
-lmult5*exp(id)/(1+exp(ib)-exp(id))-lmult13*exp(id); 
-lmult14*theta+lmult15*exp(iL)-lmult18*sig*exp(y+iL+w-z)/exp(ib); 
-lmult12*exp(ir); 
-lmult3+lmult7*nbeta*exp(ib)*exp(lambda(+1)-lambda-pi(+1))-lmult7(-1)*exp(ib(-1))*exp(lambda-lambda(-1)-pi)+lmult18*rot*nbeta*exp(lambda(+1)-lambda)*(exp(pi(+1))-1)*exp(pi(+1))*exp(y(+1))-lmult18(-1)*rot*exp(lambda-lambda(-1))*(exp(pi)-1)*exp(pi)*exp(y); 
-lmult8; 
-exp(n)^(1+chi)-lmult6*chi-lmult14*theta-lmult16*exp(z+n)*(1-exp(del)); 
lmult7(-1)*exp(ib(-1))*exp(lambda-lambda(-1)-pi)-lmult17*exp(y)*rot*(exp(pi)-1)*exp(pi)+lmult18*(rot*exp(pi)*exp(pi+y)+rot*(exp(pi)-1)*exp(pi+y))+1/nbeta*lmult18(-1)*(-rot*nbeta*exp(lambda-lambda(-1))*exp(pi)^2*exp(y)-rot*nbeta*exp(lambda-lambda(-1))*(exp(pi)-1)*exp(pi)*exp(y)); 
lmult10; 
lmult11-lmult17*exp(cd)*rcp; 
lmult10+lmult11*(ATC/exp(vel)*(exp(vel)/exp(vstar)-1-vel+vstar)-ATC/exp(vel)*(exp(vel)/exp(vstar)-1))-lmult12*exp(ib)*ATC-lmult13*exp(ib)*ATC/exp(vel); 
lmult6-lmult14*theta-lmult18*sig*exp(y+iL+w-z)/exp(ib); 
lmult16*exp(y)+lmult17*exp(y)*(1-1/2*rot*(exp(pi)-1)^2)+lmult18*(rot*(exp(pi)-1)*exp(pi+y)-(sub*(1-sig)*exp(y)+sig*exp(y+iL+w-z))/exp(ib))-lmult18(-1)*rot*exp(lambda-lambda(-1))*(exp(pi)-1)*exp(pi)*exp(y); 
 
//checks 
chk1 = n - log(ns); 
chk2 = w - log(ws); 
chk3 = pi - log(pis); 
chk4 = id - log(ids); 
chk5 = cm - log(cms); 
chk6 = vel - vels; 
chk7 = ib - log(ibs); 
chk8 = del - log(dels); 
def=exp(del); 
spread = 400*(exp(ib)-exp(ir));
end; 
 
//INITIAL VALUES. 
initval; 
cm = log(cms); cd = log(cds); cc = log(ccs); 
y = log(ns*(1-dels)); n = log(ns); w = log(ws); lambda = log(lambdas);  
ib=log(ibs); id=log(ids);iL = log(iLs); pi = log(pis);  
res = log(ress); dep = cd; del=log(dels);  
tau = taus; vel = vels; 
 
Util = phim*cm+phid*cd+phic*cc - (1/(1+chi))*ns^(1+chi); Welf = Util/(1-nbeta); 
z  = 0.0; 
eps=0; epsd=0; 
def=dels; chk1=0; chk2=0; chk3=0; chk4=0; chk5=0; chk6=0; chk7=0; chk8=0; 
 
lmult3 = lmult3_SS; 
lmult4 = lmult4_SS; 
lmult5 = lmult5_SS; 
lmult6 = lmult6_SS; 
lmult7 = lmult7_SS; 
lmult8 = lmult8_SS; 
lmult9 = lmult9_SS; 
lmult10 = lmult10_SS; 
lmult11 = lmult11_SS; 
lmult12 = lmult12_SS; 
lmult13 = lmult13_SS; 
lmult14 = lmult14_SS; 
lmult15 = lmult15_SS; 
lmult16 = lmult16_SS; 
lmult17 = lmult17_SS; 
lmult18 = lmult18_SS; 
end; 
 
steady; 
check; 
 
shocks; 
var eps = 0.000074; 
var epsd = 0.06; 
end; 
     
//SIMULATIONS 
stoch_simul(IRF=0,order=1) ib, id, iL, ir, pi, cm, n; 
//stoch_simul(IRF=0,order=2) Welf, Util; 
